import { useContext, useEffect, useRef } from 'react';
import Context from '../Context';
import { useForceUpdate, useIsomorphicLayoutEffect } from '../utils';

var isFunction = function isFunction(f) {
  return typeof f === 'function';
};

var defaultEqualityFn = function defaultEqualityFn(a, b) {
  return a === b;
};

export var useSelector = function useSelector(selector, eqlFn, options) {
  var _ref = options || !isFunction(eqlFn) && eqlFn || {},
      _ref$equalityFn = _ref.equalityFn,
      equalityFn = _ref$equalityFn === void 0 ? isFunction(eqlFn) ? eqlFn : defaultEqualityFn : _ref$equalityFn,
      _ref$customContext = _ref.customContext,
      customContext = _ref$customContext === void 0 ? Context : _ref$customContext;

  var forceUpdate = useForceUpdate();

  var _useContext = useContext(customContext),
      state = _useContext.state,
      subscribe = _useContext.subscribe;

  var selected = selector(state);
  var last = useRef(null);
  useIsomorphicLayoutEffect(function () {
    last.current = {
      equalityFn: equalityFn,
      selector: selector,
      state: state,
      selected: selected
    };
  });
  useEffect(function () {
    var callback = function callback(nextState) {
      if (last.current.state === nextState) return;
      var changed;

      try {
        changed = !last.current.equalityFn(last.current.selected, last.current.selector(nextState));
      } catch (e) {
        changed = true; // stale props or some other reason
      }

      if (changed) {
        last.current.state = nextState;
        forceUpdate();
      }
    }; // run once in case the state is already changed


    forceUpdate();
    return subscribe(callback);
  }, [subscribe, forceUpdate]);
  return selected;
};